﻿#ifndef TEMPLATECLASS1A_H
#define TEMPLATECLASS1A_H

#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include <tdatabaseutil/tsqlselectorv2.h>

class QWidget;
class TTreeView;
class TSearchEntry;
class TSplitter;

class TopTemplateClass1A : public TopClassAbs
{
    Q_OBJECT
public:
    explicit TopTemplateClass1A(const QString &iModuleName = QString(""),
                                const QVariantMap &iUrlPars = QVariantMap(),
                                QWidget *iParent = nullptr);
    ~TopTemplateClass1A();

    enum class NaviType {
        None,
        CategoryTreeView,
        UiLoader
    };

public slots:
    void refresh();
    TTreeView *treeView() const;
    QStringList queryFields() const;
    QVariantList selectedItems() const;
    QVariantList selectedDataMaps() const;
    void loadTreeData(const QVariantList &iTreeData,
                      const QString &iChildKey = QString("CHILDREN"));
    Q_DECL_DEPRECATED void loadTableData(const QVariantList &iTableData,
                                         const QString &iGroupKey,
                                         const QString &iParentKey,
                                         const QString &iSeqKey);
    void loadDataByGrouping(const QVariantList &iTableData,
                            const QString &iGroupKey,
                            const QString &iSeqKey);
    void loadDataByParent(const QVariantList &iTableData,
                          const QString &iParentKey,
                          const QString &iChildKey,
                          const QString &iSeqKey);
private slots:
    void onSelectionChanged();
    void onCategoryViewDataChanged(const QVariantList iDataLst);
    void onSearch(const QString &iText, const QVariant &iFields);
    void onNaviReset();

private:
    void initUi();
    void initNavi();
    NaviType str2NaviType(const QString &iType);
    QVariantList parseCategoryConfig();
    QString uiloaderWhere() const;
    QVariantList buildTree(const QVariantList &iDataLst,
                           const QString &iParentKey,
                           const QString &iChildKey,
                           const QString &iSeqKey);
private:
    TSqlWhereCompsiteV2 mCategoryWhere;
    NaviType mNaviType;
    QWidget *mNaviView = nullptr;
    TSplitter *mBodySplitter = nullptr;
    TSplitter *mHSplitter = nullptr;
    QWidget *mBodyWgt = nullptr;
    TTreeView *mTreeView = nullptr;
    TSearchEntry *mSearchEntry = nullptr;
    QStringList mSearchCols;
    TopClassTableConf mTreeConf;
    QVariantList mSelectedLst;
};

#endif // TEMPLATECLASS1A_H
