﻿#ifndef TEMPLATECLASS4_H
#define TEMPLATECLASS4_H

#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include <tdatabaseutil/tsqlselectorv2.h>

class TTableView;
class TSplitter;
class TUiLoader;
class TSearchEntry;
class TPageTool;
class TSimplePageTool;
class TAdvancedQuery;
class TTabWidget;
class TCategoryTreeView;

class TopTemplateClass4 : public TopClassAbs
{
    Q_OBJECT
public:
    explicit TopTemplateClass4(const QString &iModuleName = QString(""),
                               const QVariantMap &iUrlPars = QVariantMap(),
                               QWidget *iParent = nullptr);
    ~TopTemplateClass4();

    enum class NaviType {
        None,
        CategoryTreeView,
        UiLoader,
        AdvancedQuery,
        NaviFilter,
        CategoryTreeViewAndAdvancedQuery,
        UiLoaderAndCategoryTreeView
    };

public slots:
    void refresh(bool iResetPage = true);
    void refreshDetail(const QString &iUid);
    QString detailUid() const;
    void setDetailUid(const QString &iUid, bool iForceRefresh = false);
    bool isDetailModified() const;
    void setIsDetailModified(bool iIsDetailModified);
    QVariantList selectedItems() const;
    QVariantList selectedDataMaps() const;
    void newItem();
    void saveItem();
    void reloadItem();
    void copyItem();
    void deleteItems(const QVariantList &iItems);
    TTableView *tableView() const;
    TUiLoader *uiLoader() const;
    QWidget *naviView() const;
    TCategoryTreeView *naviTreeView() const;
    TUiLoader *naviLoader() const;
    TUiLoader *sumInfoLoader() const;
    TSqlSelectorV2 getSelector(bool iResetPage);
    QString getSql();
    QString getWhere();
    void focusOnObj(QObject *iObj);
signals:
    void isDetailModifiedChanged(bool iIsDetailModified);
    void detailUidChanged(const QString &iDetailUid);
    void detailSaved(const QString &iUid);
    void selectionChanged();
    void viewConfChanged();
private slots:
    void onSelectionChanged();
    void onPageChanged();
    void onCategoryViewDataChanged(const QVariantList iDataLst);
    void onDetailDataChanged();
    void onDetailSaved(const QString &iUid);
    void onDetailChanged(bool iIsDetailModified);
    void onViewConfChanged();
    void onAddActionTriggered();
    void onClearActionTriggered();
    void onSaveAsActionTriggered();
    void onOpenActtionTriggered();
    void onSaveAsCommonQueryActionTriggered();
    void onCommonQueryTableDoubleClicked(const QModelIndex &index);
    void onDeleleCommonQueryActtionTriggered();
    void onNaviReset();
    void onSumInfoDataChanged();
private:
    void initUi();
    void initNaviView();
    void initAdvanceCfgMap();
    QVariantMap getDefaultQueryDataFromCfgMap();
    QVariantList getAdvancedQueryFieldList();
    void initCommonQueryMap();
    TAdvancedQuery *findAdvancedQuery();
    TTableView *findCommonQueryTable();
    TTabWidget *findAdvancedQueryTabWidget();
    QVariantList parseCategoryConfig();
    NaviType str2NaviType(const QString &iType);
    QString uiloaderWhere();
private:
    TSplitter *mBodySplitter = nullptr;
    QWidget *mBodyWgt = nullptr;
    TSplitter *mMgtSplitter = nullptr;
    TTableView *mTableView = nullptr;
    TSearchEntry *mSearchEntry = nullptr;
    TPageTool *mPageTool = nullptr;
    TSimplePageTool *mSimplePageTool = nullptr;
    TUiLoader *mDetailView = nullptr;
    TUiLoader *mNaviViewUi = nullptr;
    TUiLoader *mSumInfoUi = nullptr;
    TCategoryTreeView *mNaviViewCat = nullptr;

    NaviType mNaviType = NaviType::CategoryTreeView;
    QWidget *mNaviView = nullptr;
    TSqlWhereCompsiteV2 mCategoryWhere;
    QVariantMap mAdvancedQueryCfgMap;
    QVariantMap mAdvancedQueryTableDataMap;
    QString mAdvancedQueryConditionFileSuffix;

    TopClassTableConf mTableConf;
    bool mIsDetailModified = false;
    QVariantList mSelectedItems;
    QString mLastDetailUid;
    QString mDetailUid;
    QTimer *mRefreshTimer = nullptr;
    bool mUiLoaderValidateOK = true;
};

#endif // TEMPLATECLASS4_H
