﻿#ifndef TEMPLATECLASS5_H
#define TEMPLATECLASS5_H

#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include <tdatabaseutil/tsqlselectorv2.h>

class TAdvancedQuery;
class TCategoryTreeView;
class TTableView;
class TSearchEntry;
class TPageTool;
class TSplitter;
class TTableView;
class TTabWidget;

class TopTemplateClass5 : public TopClassAbs
{
    Q_OBJECT
public:
    explicit TopTemplateClass5(const QString &iModuleName = QString(""),
                               const QVariantMap &iUrlPars = QVariantMap(),
                               QWidget *iParent = nullptr);
    ~TopTemplateClass5();

    enum class NaviType {
        None,
        CategoryTreeView,
        UiLoader,
        AdvancedQuery,
        NaviFilter,
        CategoryTreeViewAndAdvancedQuery,
        UiLoaderAndCategoryTreeView
    };

public slots:
    void refreshMaster(bool iResetPage = true);
    void refreshSlave(bool iResetPage = true);
    void refreshDetail();
    TTableView *masterView() const;
    TTableView *slaveView() const;
    const QStringList masterQueryFields() const;
    const QStringList slaveQueryFields() const;
    QVariantList selectedMasterItems() const;
    QVariantList selectedMasterDataMaps() const;
    QVariantList selectedSlaveItems() const;
    QVariantList selectedSlaveDataMaps() const;
    void loadTreeData(const QVariantMap &iTreeData,
                      const QString &iSlavesKey = QString("data"));
    void loadTableData(const QVariantList &iTableData,
                       const QString &iSlavesKey = QString("data"));
    void loadMasterData(const QVariantList &iMasterData, int iRowCount = -1);
    void insertSlaveData(const QString &iKey, const QVariant &iValue);
    void insertSlaveCount(const QString &iKey, int iCount);
    TUiLoader *uiLoader() const;
    void selectSlaveRow(const QVariant &iKeyVar);
    bool isDetailModified() const;
    void setIsDetailModified(bool iIsDetailModified);
    QString naviWhere();
signals:
    void isDetailModifiedChanged(bool iIsDetailModified);
private slots:
    void onMasterSelectionChanged();
    void onMasterPageChanged();
    void onSlaveSelectionChanged();
    void onSlavePageChanged();
    void onDetailChanged(bool iIsDetailModified);
    void onCategoryViewDataChanged(const QVariantList iDataLst);
    void onAddActionTriggered();
    void onClearActionTriggered();
    void onSaveAsActionTriggered();
    void onOpenActtionTriggered();
    void onSaveAsCommonQueryActionTriggered();
    void onCommonQueryTableDoubleClicked(const QModelIndex &index);
    void onDeleleCommonQueryActtionTriggered();
    void onNaviReset();
private:
    void initUi();
    void initNaviView();
    void initAdvanceCfgMap();
    QVariantMap getDefaultQueryDataFromCfgMap();
    QVariantList getAdvancedQueryFieldList();
    void initCommonQueryMap();
    TAdvancedQuery *findAdvancedQuery();
    TTableView *findCommonQueryTable();
    TTabWidget *findAdvancedQueryTabWidget();
    QVariantList parseCategoryConfig();
    NaviType str2NaviType(const QString &iType);
    QString uiloaderWhere() const;
    void resizeSplitterByRatio();
private:
    TSplitter *mBodySplitter = nullptr;
    TSplitter *mLeftSplitter = nullptr;
    TTableView *mMasterView = nullptr;
    TTableView *mSlaveView = nullptr;
    TSearchEntry *mMasterSearchEntry = nullptr;
    TPageTool *mMasterPageTool = nullptr;
    TSearchEntry *mSlaveSearchEntry = nullptr;
    TPageTool *mSlavePageTool = nullptr;
    TUiLoader *mUiLoader = nullptr;
    TUiLoader *mNaviViewUi = nullptr;
    TCategoryTreeView *mNaviViewCat = nullptr;

    NaviType mNaviType = NaviType::None;
    QWidget *mNaviView = nullptr;
    TSqlWhereCompsiteV2 mCategoryWhere;
    QVariantMap mAdvancedQueryCfgMap;
    QVariantMap mAdvancedQueryTableDataMap;
    QString mAdvancedQueryConditionFileSuffix;

    TopClassTableConf mMasterTableConf;
    TopClassTableConf mSlaveTableConf;
    QVariantList mSelectedMasterItems;
    QVariantList mSelectedSlaveItems;
    QVariantList mMasterData;
    QVariantMap mMasterSlaveDataMap;
    QMap<QString, int> mMasterSlaveCount;
    bool mIsDetailModified = false;
};

#endif // TEMPLATECLASS5_H
