﻿#ifndef TEMPLATECLASS8_H
#define TEMPLATECLASS8_H

#if _MSC_VER >= 1600
#pragma execution_character_set("utf-8")
#endif

#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <QtPrintSupport/QPrinter>

class TTableView;
class TSplitter;
class TUiLoader;
class TSearchEntry;
class TPageTool;
class TAdvancedQuery;
class TTabWidget;
class TCategoryTreeView;
class TChartWidget;
class QVBoxLayout;
class TScrollArea;
class TComboBox;

class TopTemplateClass8 : public TopClassAbs
{
    Q_OBJECT
public:
    explicit TopTemplateClass8(const QString &iModuleName = QString(""),
                               const QVariantMap &iUrlPars = QVariantMap(),
                               QWidget *iParent = nullptr);
    ~TopTemplateClass8();

    enum class NaviType {
        None,
        CategoryTreeView,
        UiLoader,
        AdvancedQuery,
        NaviFilter,
        CategoryTreeViewAndAdvancedQuery,
        UiLoaderAndCategoryTreeView
    };

public slots:
    void refresh(bool iResetPage = true);
    void refreshChart();
    QString chartId() const;
    void setChartId(const QString &iId);
    QString chartName() const;
    void setChartName(const QString &iName);
    QVariantList selectedItems() const;
    QVariantList selectedDataMaps() const;
    void deleteItems(const QVariantList &iItems);
    TTableView *tableView() const;
    QWidget *naviView() const;
    TCategoryTreeView *naviTreeView() const;
    TUiLoader *naviLoader() const;
    void setFilterData(const QVariantMap &iDataMap);
    void onDoubleClicked(); // 图表双击
    void onClear();
    void onExportPdfClicked(const QString &iPageSize = "A4"); // 导出PDF
    void onExportExcelClicked(); // 导出Excel
    void setChartVisibleData(const QVariantMap &iDataMap);
    QVariantMap chartVisibleData();
signals:
    void selectionChanged();
    void viewConfChanged();
private slots:
    void onSelectionChanged();
    void onPageChanged();
    void onCategoryViewDataChanged(const QVariantList iDataLst);
    void onViewConfChanged();
    void onAddActionTriggered();
    void onClearActionTriggered();
    void onSaveAsActionTriggered();
    void onOpenActtionTriggered();
    void onSaveAsCommonQueryActionTriggered();
    void onCommonQueryTableDoubleClicked(const QModelIndex &index);
    void onDeleleCommonQueryActtionTriggered();
    void onNaviReset();
private:
    void initUi();
    void initNaviView();
    void initAdvanceCfgMap();
    QVariantMap getDefaultQueryDataFromCfgMap();
    QVariantList getAdvancedQueryFieldList();
    void initCommonQueryMap();
    TAdvancedQuery *findAdvancedQuery();
    TTableView *findCommonQueryTable();
    TTabWidget *findAdvancedQueryTabWidget();
    QVariantList parseCategoryConfig();
    NaviType str2NaviType(const QString &iType);
    QString uiloaderWhere() const;
    void dynamicGenerateChart(const int &pId, const QString &funcJsonData, const QVariantMap &packMap, const QString &wgtType);
    void configGenerateChart(const int &pId, const QString &funcJsonData, const QVariantMap &packMap, const QString &wgtType);
    void generateUiLoader(const int &pId, const QString &funcJsonData, const QVariantMap &packMap, const QString &wgtType);
    QVariantList listPageType(); //得到导出PDF的纸张类型
    QPagedPaintDevice::PageSize pageTypeToEnum(const QString &pageSize); //纸张类型转Enum
    int calcTreeDepth(const QVariantList &iDataList);
    int calcTreeDepth(const QVariantMap &iDataMap);
    int calcTreeLeafSize(const QVariantMap &iDataMap);
private:
    TSplitter *mBodySplitter = nullptr;
    QWidget *mBodyWgt = nullptr;
    TSplitter *mMgtSplitter = nullptr;
    TTableView *mTableView = nullptr;
    TSearchEntry *mSearchEntry = nullptr;
    TPageTool *mPageTool = nullptr;
    TUiLoader *mNaviViewUi = nullptr;
    TCategoryTreeView *mNaviViewCat = nullptr;

    NaviType mNaviType = NaviType::None;
    QWidget *mNaviView = nullptr;
    TSqlWhereCompsiteV2 mCategoryWhere;
    QVariantMap mAdvancedQueryCfgMap;
    QVariantMap mAdvancedQueryTableDataMap;
    QString mAdvancedQueryConditionFileSuffix;

    QWidget* mChartListWgt = nullptr;
    QVBoxLayout* mUiLoaderLayout = nullptr;
    QMap<QString, TChartWidget*> mChartMap;
    QMap<TChartWidget*, QString> mChartParameterRemarkMap; //TChartWidget对应的graphId
    QMap<QString, TUiLoader*> mUiLoaderMap;
    QMap<TUiLoader*, QString> mUiLoaderParamRemarkMap;
    TScrollArea *mScrollArea = nullptr;
    QVariantMap mChartVisibleData;
    TComboBox* mSizeComboBox = nullptr;

    TopClassTableConf mTableConf;
    QString mChartId;
    QString mChartName;
    QString mGraphLayout;
    QTimer *mRefreshTimer = nullptr;
    QVariantMap mFilterData;
};

#endif // TEMPLATECLASS8_H
